import 'dart:ui' show Canvas, Image;

import 'package:composition_rendering/core.dart';
import 'package:composition_rendering/scene.dart';
import 'package:composition_rendering/systems.dart';
import 'package:flame/components.dart' show Component;
import 'package:flame/flame.dart';

class FlamePluginComponent extends Component with Registry {
  static final PluginSystemRegistry registry = PluginSystemRegistry();

  late final Scene _scene;

  FlamePluginComponent() {
    SystemRegistry.instance = registry;
    changePriorityWithoutResorting(50);
  }

  set scene(Scene scene) {
    _scene = scene;
  }

  @override
  Future<void>? onLoad() {
    _scene.init();
  }

  void onResize(Vector2 canvasSize) {
    parameters.setViewSize(canvasSize.x, canvasSize.y);
  }

  @override
  void update(double dt) {
    systems.updateSystem.update(dt);
    _scene.updateScene(dt);
  }

  @override
  void render(Canvas canvas) {
    systems.renderSystem.render(canvas);
  }
}

class PluginSystemRegistry extends SystemRegistry {
  PluginSystemRegistry() : super(textureSystem: PluginTextureSystem(), parameters: Parameters());
}

class PluginTextureSystem extends TextureSystem {
  @override
  Future<Image> load(String src) async {
    return await Flame.images.load(src);
  }

  @override
  Image getImageFromCache(String fileName) {
    return Flame.images.fromCache(fileName);
  }
}
